@@ -2,7 +2,7 @@
 #this repository contains the full copyright notices and license terms.
 from trytond.model import ModelSQL, fields
 from trytond.pyson import Eval, Or
-from trytond.backend import TableHandler
+from trytond import backend
 from trytond.transaction import Transaction
 from trytond.pool import PoolMeta
 
@@ -18,8 +18,8 @@ class Category:
     account_expense = fields.Property(fields.Many2One('account.account',
             'Account Expense', domain=[
                 ('kind', '=', 'expense'),
-                ('company', '=', Eval('context', {}).get('company', 0)),
-                ], on_change=['account_expense'],
+                ('company', '=', Eval('context', {}).get('company', -1)),
+                ],
             states={
                 'invisible': (~Eval('context', {}).get('company')
                     | Eval('account_parent')),
@@ -28,8 +28,8 @@ class Category:
     account_revenue = fields.Property(fields.Many2One('account.account',
             'Account Revenue', domain=[
                 ('kind', '=', 'revenue'),
-                ('company', '=', Eval('context', {}).get('company', 0)),
-                ], on_change=['account_revenue'],
+                ('company', '=', Eval('context', {}).get('company', -1)),
+                ],
             states={
                 'invisible': (~Eval('context', {}).get('company')
                     | Eval('account_parent')),
@@ -95,6 +95,7 @@ class Category:
         else:
             return [x.id for x in getattr(self, name[:-5])]
 
+    @fields.depends('account_expense')
     def on_change_account_expense(self):
         supplier_taxes = []
         result = {
@@ -104,6 +105,7 @@ class Category:
             supplier_taxes.extend(tax.id for tax in self.account_expense.taxes)
         return result
 
+    @fields.depends('account_revenue')
     def on_change_account_revenue(self):
         customer_taxes = []
         result = {
@@ -125,6 +127,7 @@ class CategoryCustomerTax(ModelSQL):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
         # Migration from 1.6 product renamed into category
         table = TableHandler(cursor, cls)
@@ -146,6 +149,7 @@ class CategorySupplierTax(ModelSQL):
 
     @classmethod
     def __register__(cls, module_name):
+        TableHandler = backend.get('TableHandler')
         cursor = Transaction().cursor
         # Migration from 1.6 product renamed into category
         table = TableHandler(cursor, cls)
@@ -163,8 +167,8 @@ class Template:
     account_expense = fields.Property(fields.Many2One('account.account',
             'Account Expense', domain=[
                 ('kind', '=', 'expense'),
-                ('company', '=', Eval('context', {}).get('company', 0)),
-                ], on_change=['account_category', 'account_expense'],
+                ('company', '=', Eval('context', {}).get('company', -1)),
+                ],
             states={
                 'invisible': (~Eval('context', {}).get('company')
                     | Eval('account_category')),
@@ -174,8 +178,8 @@ class Template:
     account_revenue = fields.Property(fields.Many2One('account.account',
             'Account Revenue', domain=[
                 ('kind', '=', 'revenue'),
-                ('company', '=', Eval('context', {}).get('company', 0)),
-                ], on_change=['account_category', 'account_revenue'],
+                ('company', '=', Eval('context', {}).get('company', -1)),
+                ],
             states={
                 'invisible': (~Eval('context', {}).get('company')
                     | Eval('account_category')),
@@ -243,6 +247,7 @@ class Template:
         else:
             return [x.id for x in getattr(self, name[:-5])]
 
+    @fields.depends('account_category', 'account_expense')
     def on_change_account_expense(self):
         result = {}
         if not self.account_category:
@@ -255,6 +260,7 @@ class Template:
                     tax.id for tax in self.account_expense.taxes)
         return result
 
+    @fields.depends('account_category', 'account_revenue')
     def on_change_account_revenue(self):
         result = {}
         if not self.account_category:
